;------------------------------------------------
; drawImage8BitHuff - copy an 8-bit huffman compressed image to the lcd gram
;   input:  DE => image data (in uncompressed 8-bit BGR format)
;           IX => 1024 bytes memory for huffman tree
;           B = file # in huffman data that has the image (not required if only 1 file compressed)
;   output: none
;   notes:  destroys AF, BC, DE, HL, IX
;           this routine is essentially Jimmy Mardell's "HuffExtr" routine modified to write directly to the 84+C gram
;------------------------------------------------
drawImage8BitHuff:
        ld      a,$20
        ld      hl,WIN_TOP
        call    lcdReg                          ; cursor to y=WIN_TOP
        inc     a                               ; A = $21
        ld      hl,WIN_LEFT
        call    lcdReg                          ; cursor to x=WIN_LEFT
        inc     a                               ; A = $22
        out     ($10),a
        out     ($10),a                         ; point lcd driver to gram
        ex      de,hl                           ; HL => image data
;        push    de
;        push    de
        push    hl
        inc     hl
        ld      a,(hl)
        cp      1
        jr      nz,MultiFile
        ld      b,0
MultiFile:
        push    bc
        inc     hl
        ld      d,(hl)
        inc     hl
        ld      e,(hl)                          ; DE = noDifChars
        inc     hl
        push    hl
        ld      l,a
        ld      h,0                             ; HL = A
        add     hl,hl                           ; HL = 2A
        ld      b,h
        ld      c,l                             ; BC = 2A
        add     hl,hl                           ; HL = 4A
        add     hl,bc                           ; HL = 6A
        ld      b,h
        ld      c,l                             ; BC = 6A
        pop     hl
        push    hl
        add     hl,bc                           ; HL -> chars
        ld      b,e
        dec     b                               ; B = noDifChars-1
        push    hl
        add     hl,de
        pop     de                              ; DE -> chars, HL -> tree, IX -> treeaddr
        ld      (treeaddr),ix
        ld      a,1
UncrunchTree:
        ld      c,a
        and     (hl)
        jr      z,NoBranch
        push    ix
        inc     ix
        inc     ix
NextTreeBit:
        ld      a,c
        rlca
        jr      nc,UncrunchTree
        inc     hl
        jr      UncrunchTree
NoBranch:
        ld      a,(de)
        inc     de
        ld      (ix),0
        ld      (ix+1),a
        inc     ix
        inc     ix
        ld      a,b
        or      a
        jr      z,TreeBuilt
        ex      (sp),hl
        push    de
        push    ix
        pop     de
        ld      (hl),d
        inc     hl
        ld      (hl),e                          ; MSB 1st order
        pop     de
        ex      (sp),hl
        inc     sp
        inc     sp
        dec     b
        jr      NextTreeBit
TreeBuilt:
        pop     ix                              ; IX -> fileInfo
        pop     af                              ; A = file no
        ld      l,a
        ld      h,0                             ; HL = A
        add     hl,hl                           ; HL = 2A
        ld      b,h
        ld      c,l                             ; BC = 2A
        add     hl,hl                           ; HL = 4A
        add     hl,bc                           ; HL = 6A
        ld      b,h
        ld      c,l                             ; BC = 6A
        pop     hl                              ; HL -> noFiles
        add     ix,bc                           ; IX -> fileInfo[file no]
        ld      d,(ix)
        ld      e,(ix+1)
        add     hl,de                           ; HL -> data
        ld      b,(ix+3)                        ; B = start bit
        ld      a,1
Shift:
        rlca
        djnz    Shift
        ld      c,a                             ; C = start bit vlaue
        ld      d,(ix+4)
        ld      e,(ix+5)                        ; DE = length of uncompressed data
;        pop     ix                              ; HL -> Data, C = bitval, DE = length
UncrunchData:
        push    de
        ld      de,(treeaddr)
CheckTree:
        ld      a,(de)                          ; cause of this, we need to store in MSB 1st order
        or      a
        jr      z,EndOfBranch
        ld      a,c
        and     (hl)
        jr      nz,RightBranch
        inc     de
        inc     de
        jr      NextDataBit
RightBranch:
        ex      de,hl
        ld      a,(hl)
        inc     hl
        ld      l,(hl)
        ld      h,a                             ; H = (HL), L = (HL+1) : MSB 1st order
        ex      de,hl
NextDataBit:
        rlc     c
        jr      nc,CheckTree
        inc     hl
        jr      CheckTree
EndOfBranch:
        inc     de
        push    hl                              ; save data ptr
        ld      a,(de)
        ld      l,a
        ld      h,0                             ; HL = 8-bit color
        ld      d,h
        ld      e,l
        add     hl,de                           ; HL = offset for colorIndex
        ld      de,colorIndex
        add     hl,de                           ; HL => 16-bit color data (high endian)
        ld      a,(hl)
        out     ($11),a                         ; send first byte
        inc     hl
        ld      a,(hl)
        out     ($11),a                         ; send second byte
        pop     hl                              ; restore data ptr
;        ld      (ix),a
;        inc     ix
        pop     de
        dec     de
        ld      a,d
        or      e
        jr      nz,UncrunchData
;        pop     de
        ret

treeaddr:
.dw     0

.end
