;================================================
; Splash84C - splash screen demo for the TI-84+C
; by James Vernon
; jamesv82@live.com.au
; www.jvti.org
;
; Feel free to use any of this code if you need it
;================================================

#define BGCOLOR $0000
#define IMG_WIDTH 180
#define IMG_HEIGHT 171
#define WIN_LEFT 70
#define WIN_RIGHT 249
#define WIN_TOP 34
#define WIN_BOTTOM 204

#include "ti84pcse.inc"                         ; thanks to Brandon Wilson

huffbuff = userMem-2-1024                       ; 1024 byte buffer for Huffman tree (not sure if this is scrap RAM??)

.org    userMem-2
.db     $EF,$69

start:
        res     apdrunning,(iy+apdflags)        ; disable apd
        ld      a,$01
        out     ($20),a                         ; run @ 15Mhz
        call    fullWindow
        call    clearFullWindow
        call    partWindow
        ld      de,image                        ; DE => image data
        call    drawImage8BitHuff
        ld      b,50
delayLoop:
        ld      de,50000
delayInnerLoop:
        dec     de
        ld      a,d
        or      e
        jr      nz,delayInnerLoop
        djnz    delayLoop
quit:
        call    fullWindow
        bcall(_DrawStatusBar)
        bcall(_maybe_ClrScrnFull)
        set     apdrunning,(iy+apdflags)        ; enable apd
        ret

;------------------------------------------------
; fullWindow - set the lcd to full window mode
;   input:  none
;   output: none
;   notes:  destroys A, C, HL
;------------------------------------------------
fullWindow:
        ld      a,$50                           ; lcd reg to mod (window_min_y)
        ld      hl,0                            ; value
        call    lcdReg                          ; set it
        inc     a                               ; A = $51 (next reg, which is window_max_y)
        ld      l,239                           ; HL = 239
        call    lcdReg
        inc     a                               ; A = window_min_x
        ld      l,0
        call    lcdReg
        inc     a                               ; A = window_max_x
        ld      hl,319
        jr      lcdReg

;------------------------------------------------
; partWindow - set the lcd to a smaller window mode, with coords set by the defines at the start of the code
;   input:  WIN_TOP, WIN_BOTTOM, WIN_LEFT, WIN_RIGHT
;   output: none
;   notes:  destroys A, C, HL
;------------------------------------------------
partWindow:
        ld      a,$50
        ld      hl,WIN_TOP
        call    lcdReg
        inc     a
        ld      hl,WIN_BOTTOM
        call    lcdReg
        inc     a
        ld      hl,WIN_LEFT
        call    lcdReg
        inc     a
        ld      hl,WIN_RIGHT
; fall through to lcdReg

;------------------------------------------------
; lcdReg - set a 16-bit lcd register
;   input:  A = register #
;           HL = 16-bit value
;   output: A & HL preserved
;   notes:  destroys C
;------------------------------------------------
lcdReg:
        out     ($10),a
        out     ($10),a
        ld      c,$11
        out     (c),h
        out     (c),l
        ret

;------------------------------------------------
; clearFullWindow - clear the lcd (in full window mode)
;   input:  #define BGCOLOR = a 16-bit BGR color to set the entire lcd to
;   output: none
;   notes:  destroys AF, C, DE, HL
;------------------------------------------------
clearFullWindow:
        ld      a,$20
        ld      hl,0
        call    lcdReg                          ; cursor to y=0
        inc     a                               ; A = $21
        call    lcdReg                          ; cursor to x=0
        inc     a                               ; A = $22
        out     ($10),a
        out     ($10),a                         ; point lcd driver to gram
        ld      de,320*240/2                    ; DE = # pixels to write / 2 (remembering, 2 bytes per pixel)
        ld      c,$11                           ; C = lcd data port
        ld      hl,BGCOLOR                      ; HL = data for pixel
clrFWinLoop:
        out     (c),h
        out     (c),l                           ; first pixel cleared
        out     (c),h
        out     (c),l                           ; second pixel cleared
        dec     de
        ld      a,d
        or      e                               ; reached 0?
        jr      nz,clrFWinLoop                  ; if not, loop
        ret

;#include "img8u.asm"                            ; routine to show 8-bit uncompressed image
#include "img8huf.asm"                          ; routine to draw 8-bit huffman compressed image

; color index table for 256 color mode
colorIndex:
#include "cindex.asm"

; image data imported here
image:
#import "ab.huf"

.end
